@@ -136,7 +136,8 @@ class BrailleGenerator(generator.Generator):
 
         doNotPresent = [pyatspi.ROLE_UNKNOWN,
                         pyatspi.ROLE_FILLER,
-                        pyatspi.ROLE_EXTENDED]
+                        pyatspi.ROLE_EXTENDED,
+                        pyatspi.ROLE_LINK]
 
         # egg-list-box, e.g. privacy panel in gnome-control-center
         if obj.parent and obj.parent.getRole() == pyatspi.ROLE_LIST_BOX:
@@ -145,10 +146,8 @@ class BrailleGenerator(generator.Generator):
         if verbosityLevel == settings.VERBOSITY_LEVEL_BRIEF:
             doNotPresent.extend([pyatspi.ROLE_ICON, pyatspi.ROLE_CANVAS])
 
-        if (role in _settingsManager.getSetting('brailleForceRoles'))\
-           or (verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE\
-               and not args.get('readingRow', False)\
-               and role not in doNotPresent):
+        if verbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE \
+           and not args.get('readingRow', False) and role not in doNotPresent:
             result.append(self.getLocalizedRoleName(obj, role))
         return result
 
@@ -305,7 +304,7 @@ class BrailleGenerator(generator.Generator):
     def _generateFocusedItem(self, obj, **args):
         result = []
         role = args.get('role', obj.getRole())
-        if role != pyatspi.ROLE_LIST:
+        if role not in [pyatspi.ROLE_LIST, pyatspi.ROLE_LIST_BOX]:
             return result
 
         try:
