@@ -392,6 +392,12 @@ INCREASE_SPEECH_PITCH = _("Increases the speech pitch.")
 #  We call it 'silencing'.
 TOGGLE_SPEECH = _("Toggles the silencing of speech.")
 
+# Translators: Orca's verbosity levels control how much (or how little)
+# Orca will speak when presenting objects as the user navigates within
+# applications and reads content. The levels can be toggled via command.
+# This string describes that command.
+TOGGLE_SPEECH_VERBOSITY = _("Toggles speech verbosity level.")
+
 # Translators: this string is associated with the keyboard shortcut to quit
 # Orca.
 QUIT_ORCA = _("Quits the screen reader")
@@ -736,14 +742,6 @@ PRESENT_INPUT_LINE = _("Presents the contents of the input line.")
 # writing functions.
 STRUCTURAL_NAVIGATION_TOGGLE = _("Toggles structural navigation keys.")
 
-# Translators: this is for navigating among anchors in a document. An anchor is
-# a named spot that one can jump to.
-ANCHOR_PREV = _("Goes to previous anchor.")
-
-# Translators: this is for navigating among anchors in a document. An anchor is
-# a named spot that one can jump to.
-ANCHOR_NEXT = _("Goes to next anchor.")
-
 # Translators: this is for navigating among blockquotes in a document.
 BLOCKQUOTE_PREV = _("Goes to previous blockquote.")
 
@@ -771,6 +769,18 @@ CHECK_BOX_NEXT = _("Goes to next check box.")
 # Translators: this is for navigating among check boxes in a document.
 CHECK_BOX_LIST = _("Displays a list of check boxes.")
 
+# Translators: this is for navigating among clickable objects in a document.
+# A "clickable" is a web element with an "onClick" handler.
+CLICKABLE_PREV = _("Goes to previous clickable.")
+
+# Translators: this is for navigating among clickable objects in a document.
+# A "clickable" is a web element with an "onClick" handler.
+CLICKABLE_NEXT = _("Goes to next clickable.")
+
+# Translators: this is for navigating among clickable objects in a document.
+# A "clickable" is a web element with an "onClick" handler.
+CLICKABLE_LIST = _("Displays a list of clickables.")
+
 # Translators: this is for navigating among combo boxes in a document.
 COMBO_BOX_PREV = _("Goes to previous combo box.")
 
@@ -819,6 +829,15 @@ HEADING_AT_LEVEL_NEXT = _("Goes to next heading at level %d.")
 # <h1> is a heading at level 1, <h2> is a heading at level 2, etc.
 HEADING_AT_LEVEL_LIST = _("Displays a list of headings at level %d.")
 
+# Translators: this is for navigating among images in a document.
+IMAGE_PREV = _("Goes to previous image.")
+
+# Translators: this is for navigating among images in a document.
+IMAGE_NEXT = _("Goes to next image.")
+
+# Translators: this is for navigating among images in a document.
+IMAGE_LIST = _("Displays a list of images.")
+
 # Translators: this is for navigating among ARIA landmarks in a document. ARIA
 # role landmarks are the W3C defined HTML tag attribute 'role' used to identify
 # important part of webpage like banners, main context, search etc.
@@ -829,6 +848,11 @@ LANDMARK_PREV = _("Goes to previous landmark.")
 # important part of webpage like banners, main context, search etc.
 LANDMARK_NEXT = _("Goes to next landmark.")
 
+# Translators: this is for navigating among ARIA landmarks in a document. ARIA
+# role landmarks are the W3C defined HTML tag attribute 'role' used to identify
+# important part of webpage like banners, main context, search etc.
+LANDMARK_LIST = _("Displays a list of landmarks.")
+
 # Translators: this is for navigating among large objects in a document.
 # A 'large object' is a logical chunk of text, such as a paragraph, a list,
 # a table, etc.
@@ -844,6 +868,12 @@ LARGE_OBJECT_NEXT = _("Goes to next large object.")
 # a table, etc.
 LARGE_OBJECT_LIST = _("Displays a list of large objects.")
 
+# Translators: this is for navigating among links in a document.
+LINK_PREV = _("Goes to previous link.")
+
+# Translators: this is for navigating among links in a document.
+LINK_NEXT = _("Goes to next link.")
+
 # Translators: this is for navigating among links in a document.
 LINK_LIST = _("Displays a list of links.")
 
@@ -933,6 +963,33 @@ TABLE_CELL_RIGHT = _("Goes right one cell.")
 # Translators: this is for navigating among table cells in a document.
 TABLE_CELL_UP = _("Goes up one cell.")
 
+# Translators: Orca has a number of commands that override the default
+# behavior within an application. For instance, on a web page, "h" moves
+# you to the next heading. What should happen when you press an "h" in
+# an entry on a web page depends: If you want to resume reading content,
+# "h" should move to the next heading; if you want to enter text, "h"
+# should not not move you to the next heading. Similarly, if you are
+# at the bottom of an entry and press Down arrow, should you leave the
+# entry? Again, it depends on if you want to resume reading content or
+# if you are editing the text in the entry. Because Orca doesn't know
+# what you want to do, it has two modes: In browse mode, Orca treats
+# key presses as commands to read the content; in focus mode, Orca treats
+# key presses as something that should be handled by the focused widget.
+# This string is associated with the Orca command to manually switch
+# between these two modes.
+TOGGLE_PRESENTATION_MODE = _("Switches between browse mode and focus mode.")
+
+# Translators: (Please see the previous, detailed translator notes about
+# Focus mode and Browse mode.) In order to minimize the amount of work Orca
+# users need to do to switch between focus mode and browse mode, Orca attempts
+# to automatically switch to the mode which is appropriate to the current
+# web element. Sometimes, however, this automatic mode switching is not what
+# the user wants. A good example being web apps which have their own keyboard
+# navigation and use interaction model. As a result, Orca has a command which
+# enables setting a "sticky" focus mode which disables all automatic toggling.
+# This string is associated with the Orca command to enable sticky focus mode.
+SET_FOCUS_MODE_STICKY = _("Enables sticky focus mode.")
+
 # Translators: this is for navigating among unvisited links in a document.
 UNVISITED_LINK_PREV = _("Goes to previous unvisited link.")
 
