@@ -39,7 +39,7 @@ import sys
 
 try:
     from gi.repository.Gio import Settings
-    a11yAppSettings = Settings('org.gnome.desktop.a11y.applications')
+    a11yAppSettings = Settings(schema_id='org.gnome.desktop.a11y.applications')
 except:
     a11yAppSettings = None
 
@@ -60,9 +60,12 @@ except:
 from . import braille
 from . import debug
 from . import event_manager
+from . import keybindings
+from . import logger
 from . import messages
 from . import notification_messages
 from . import orca_state
+from . import orca_platform
 from . import script_manager
 from . import settings
 from . import settings_manager
@@ -73,6 +76,7 @@ from .input_event import KeyboardEvent
 _eventManager = event_manager.getManager()
 _scriptManager = script_manager.getManager()
 _settingsManager = settings_manager.getManager()
+_logger = logger.getLogger()
 
 try:
     # If we don't have an active desktop, we will get a RuntimeError.
@@ -92,6 +96,9 @@ def onEnabledChanged(gsetting, key):
 def getSettingsManager():
     return _settingsManager
 
+def getLogger():
+    return _logger
+
 EXIT_CODE_HANG = 50
 
 # The user-settings module (see loadUserSettings).
@@ -166,11 +173,15 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
             appname = "'" + app.name + "'"
         except:
             appname = "None"
+        try:
+            name = orca_state.locusOfFocus.name
+            rolename = orca_state.locusOfFocus.getRoleName()
+        except:
+            name = "Error"
+            rolename = "Error"
         debug.println(debug.LEVEL_FINE,
                       "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
-                      % (appname,
-                         orca_state.locusOfFocus.name,
-                         orca_state.locusOfFocus.getRoleName()))
+                      % (appname, name, rolename))
 
         if event:
             debug.println(debug.LEVEL_FINE,
@@ -206,12 +217,17 @@ def _processKeyboardEvent(event):
     """
     global _orcaModifierPressed
 
-    # Weed out duplicate and otherwise bogus events.
     keyboardEvent = KeyboardEvent(event)
     debug.println(debug.LEVEL_FINE, keyboardEvent.toString())
-    if keyboardEvent.ignoreDueToTimestamp():
-        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO TIMESTAMP")
-        return
+
+    # Weed out duplicate and otherwise bogus events.
+    # TODO - JD: Be sure these are the right values to return
+    if not keyboardEvent.timestamp:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: NO TIMESTAMP")
+        return False
+    if keyboardEvent == orca_state.lastInputEvent:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT: DUPLICATE")
+        return False
 
     # Figure out what we've got.
     isOrcaModifier = keyboardEvent.isOrcaModifier()
@@ -219,10 +235,9 @@ def _processKeyboardEvent(event):
     if isOrcaModifier:
         _orcaModifierPressed = isPressedEvent
     if _orcaModifierPressed:
-        keyboardEvent.modifiers |= settings.ORCA_MODIFIER_MASK
+        keyboardEvent.modifiers |= keybindings.ORCA_MODIFIER_MASK
 
     # Update our state.
-    orca_state.lastInputEventTimestamp = event.timestamp
     orca_state.lastInputEvent = keyboardEvent
     if not keyboardEvent.isModifierKey():
         keyboardEvent.setClickCount()
@@ -230,12 +245,15 @@ def _processKeyboardEvent(event):
 
     # Echo it based on what it is and the user's settings.
     script = orca_state.activeScript
-    if script:
-        if isPressedEvent:
-            script.presentationInterrupt()
-        keyboardEvent.present()
-        if keyboardEvent.isModifierKey() and not isOrcaModifier:
-            return False
+    if not script:
+        debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO NO SCRIPT")
+        return False
+
+    if isPressedEvent:
+        script.presentationInterrupt()
+    script.presentKeyboardEvent(keyboardEvent)
+    if keyboardEvent.isModifierKey() and not isOrcaModifier:
+        return False
  
     # Special modes.
     if not isPressedEvent and keyboardEvent.event_string == "Escape":
@@ -244,13 +262,12 @@ def _processKeyboardEvent(event):
         if keyboardEvent.event_string == "F1":
             orca_state.learnModeEnabled = False
             return helpForOrca()
-        if keyboardEvent.event_string in ["1", "2"]:
-            orca_state.learnModeEnabled = False
+        if isPressedEvent and keyboardEvent.event_string in ["F2", "F3"]:
             return script.listOrcaShortcuts(keyboardEvent)
     if orca_state.capturingKeys:
         return False
     if notification_messages.listNotificationMessagesModeEnabled:
-        return notification_messages.listNotificationMessages(keyboardEvent)
+        return notification_messages.listNotificationMessages(script, keyboardEvent)
 
     # See if the event manager wants it (i.e. it is bound to a command.
     if _eventManager.processKeyboardEvent(keyboardEvent):
@@ -426,6 +443,8 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
+    _settingsManager.loadAppSettings(script)
+
     if _settingsManager.getSetting('enableSpeech'):
         try:
             speech.init()
@@ -475,6 +494,26 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
 
     return True
 
+def _showPreferencesUI(script, prefs):
+    if orca_state.orcaOS:
+        orca_state.orcaOS.showGUI()
+        return
+
+    try:
+        module = importlib.import_module('.orca_gui_prefs', 'orca')
+    except:
+        debug.printException(debug.LEVEL_SEVERE)
+        return
+
+    uiFile = os.path.join(orca_platform.datadir,
+                          orca_platform.package,
+                          "ui",
+                          "orca-setup.ui")
+
+    orca_state.orcaOS = module.OrcaSetupGUI(uiFile, "orcaSetupWindow", prefs)
+    orca_state.orcaOS.init(script)
+    orca_state.orcaOS.showGUI()
+
 def showAppPreferencesGUI(script=None, inputEvent=None):
     """Displays the user interace to configure the settings for a
     specific applications within Orca and set up those app-specific
@@ -483,11 +522,12 @@ def showAppPreferencesGUI(script=None, inputEvent=None):
     Returns True to indicate the input event has been consumed.
     """
 
-    try:
-        module = importlib.import_module('.app_gui_prefs', 'orca')
-        module.showPreferencesUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
+    prefs = {}
+    for key in settings.userCustomizableSettings:
+        prefs[key] = _settingsManager.getSetting(key)
+
+    script = script or orca_state.activeScript
+    _showPreferencesUI(script, prefs)
 
     return True
 
@@ -498,11 +538,9 @@ def showPreferencesGUI(script=None, inputEvent=None):
     Returns True to indicate the input event has been consumed.
     """
 
-    try:
-        module = importlib.import_module('.orca_gui_prefs', 'orca')
-        module.showPreferencesUI()
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
+    prefs = _settingsManager.getGeneralSettings(_settingsManager.profile)
+    script = _scriptManager.getDefaultScript()
+    _showPreferencesUI(script, prefs)
 
     return True
 
@@ -585,7 +623,7 @@ def init(registry):
 
     return True
 
-def start(registry):
+def start(registry, cacheValues):
     """Starts Orca.
     """
 
@@ -603,11 +641,11 @@ def start(registry):
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
 
-    if settings.cacheValues:
+    if cacheValues:
         pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
     debug.println(debug.LEVEL_FINEST, 'INFO: Orca starting registry')
-    registry.start(gil=settings.useGILIdleHandler)
+    registry.start(gil=False)
 
 def die(exitCode=1):
     pid = os.getpid()
@@ -650,8 +688,8 @@ def shutdown(script=None, inputEvent=None):
 
     orca_state.activeScript.presentMessage(messages.STOP_ORCA)
 
-    _eventManager.deactivate()
     _scriptManager.deactivate()
+    _eventManager.deactivate()
 
     # Shutdown all the other support.
     #
@@ -723,7 +761,7 @@ def abortOnSignal(signum, frame):
                   % signum)
     die(signum)
 
-def main():
+def main(cacheValues=True):
     """The main entry point for Orca.  The exit codes for Orca will
     loosely be based on signals, where the exit code will be the
     signal used to terminate Orca (if a signal was used).  Otherwise,
@@ -751,7 +789,9 @@ def main():
         message = messages.START_ORCA
         if not _settingsManager.getSetting('onlySpeakDisplayedText'):
             speech.speak(message, settings.voices.get(settings.SYSTEM_VOICE))
-        braille.displayMessage(message)
+        if _settingsManager.getSetting('enableBraille') \
+           or _settingsManager.getSetting('enableBrailleMonitor'):
+            braille.displayMessage(message)
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
@@ -762,7 +802,7 @@ def main():
             setLocusOfFocus(None, window)
 
     try:
-        start(pyatspi.Registry) # waits until we stop the registry
+        start(pyatspi.Registry, cacheValues) # waits until we stop the registry
     except:
         die(EXIT_CODE_HANG)
     return 0
