@@ -106,7 +106,6 @@ class Script:
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
         self.brailleBindings = self.getBrailleBindings()
-        self.app_pronunciation_dict = self.getPronunciations()
 
         self.formatting = self.getFormatting()
         self.brailleGenerator = self.getBrailleGenerator()
@@ -115,12 +114,14 @@ class Script:
         self.eventCache = {}
         self.whereAmI = self.getWhereAmI()
         self.bookmarks = self.getBookmarks()
+        self.spellcheck = self.getSpellCheck()
         self.voices = settings.voices
         self.tutorialGenerator = self.getTutorialGenerator()
 
         self.flatReviewContextClass = flat_review.Context
 
         self.findCommandRun = False
+        self._lastCommandWasStructNav = False
 
         debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
 
@@ -181,15 +182,6 @@ class Script:
         """
         return {}
 
-    def getPronunciations(self):
-        """Defines the application specific pronunciations for this script.
-
-        Returns a dictionary where the keys are the actual text strings and
-        the values are the replacement strings that are spoken instead.
-        """
-
-        return {}
-
     def getBrailleCommandsForInputHandler(self, inputEventHandler):
         """Returns a list of BrlTTY commands (they're in braille.py) that
         match the given inputEventHandler passed as argument.
@@ -228,6 +220,10 @@ class Script:
         """
         return None
 
+    def getSpellCheck(self):
+        """Returns the spellcheck support for this script."""
+        return None
+
     def getUtilities(self):
         """Returns the utilites for this script.
         """
@@ -278,48 +274,10 @@ class Script:
         """
         return None
 
-    def setAppPreferences(self, prefs):
-        """Write out the application specific preferences lines and set the
-        new values.
+    def getPreferencesFromGUI(self):
+        """Returns a dictionary with the app-specific preferences."""
 
-        Arguments:
-        - prefs: file handle for application preferences.
-        """
-        pass
-
-    def overrideAppKeyBindings(self, script, keyBindings):
-        """Allow for the customization of application specific key bindings.
-
-        Arguments:
-        - script: the application script.
-        - keyBindings: the set of key bindings for this script.
-        """
-
-        return keyBindings
-
-    def overridePronunciations(self, script, pronunciations):
-        """Allow for the customization of application specific pronunciations.
-
-        Arguments:
-        - script: the application script.
-        - pronunciations: the dictionary of pronunciations for this script.
-        """
-
-        return pronunciations
-
-    def getAppState(self):
-        """Returns an object that can be passed to setAppState.  This
-        object will be used by setAppState to restore any state
-        information that was being maintained by the script."""
-        return None
-
-    def setAppState(self, appState):
-        """Sets the application state using the given appState object.
-
-        Arguments:
-        - appState: an object obtained from getAppState
-        """
-        return
+        return {}
 
     def registerEventListeners(self):
         """Tells the event manager to start listening for all the event types
@@ -383,11 +341,6 @@ class Script:
         - event: the Event
         """
 
-        if not event.source:
-            msg = 'script.processObjectEvent: event.source went away'
-            debug.println(debug.LEVEL_FINE, msg)
-            return
-
         try:
             role = event.source.getRole()
         except (LookupError, RuntimeError):
@@ -442,10 +395,10 @@ class Script:
         if not cachedEvent or cachedEvent == event:
             return False
 
-        focus    = ["focus:", "object:state-changed:focused"]
+        focus    = ["object:state-changed:focused"]
         typing   = ["object:text-changed:insert", "object:text-changed:delete"]
         arrowing = ["object:text-caret-moved", "object:text-selection-changed",
-                    "object:selection-changed"]
+                    "object:selection-changed", "object:active-descendant-changed"]
 
         skip = False
         if (event.type in arrowing or event.type in typing) \
@@ -503,18 +456,23 @@ class Script:
             user_bindings = user_bindings_map["default"]
 
         consumes = False
+        self._lastCommandWasStructNav = False
         if user_bindings:
             handler = user_bindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         return consumes
