@@ -26,10 +26,7 @@
 #
 # pylint: disable-msg=W0142
 
-"""Provides an Orca speech server for Speech Dispatcher backend.
-
-NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
-"""
+"""Provides an Orca speech server for Speech Dispatcher backend."""
 
 __id__ = "$Id$"
 __version__   = "$Revision$"
@@ -38,17 +35,19 @@ __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
 
-from gi.repository import GObject
+from gi.repository import GLib
 import re
+import time
 
 from . import chnames
 from . import debug
+from . import guilabels
+from . import messages
 from . import speechserver
 from . import settings
 from . import orca_state
 from . import punctuation_settings
 from .acss import ACSS
-from .orca_i18n import _
 
 try:
     import speechd
@@ -72,28 +71,10 @@ class SpeechServer(speechserver.SpeechServer):
     _active_servers = {}
     
     DEFAULT_SERVER_ID = 'default'
-    
-    # Translators: "Default Synthesizer" will appear in the list of available
-    # speech engines as a special item.  It refers to the default engine
-    # configured within the speech subsystem.  Apart from this item, the user
-    # will have a chance to select a particular speech engine by its real
-    # name, such as Festival, IBMTTS, etc.
-    #
-    _SERVER_NAMES = {DEFAULT_SERVER_ID: _("Default Synthesizer")}
-    
-    KEY_NAMES = {
-        '_':     'underscore',
-        ' ':     'space',
-        'space': 'space',
-        '"':     'double-quote',
-        }
-
+    _SERVER_NAMES = {DEFAULT_SERVER_ID: guilabels.DEFAULT_SYNTHESIZER}
 
     def getFactoryName():
-        # Translators: this is the name of a speech synthesis system
-        # called "Speech Dispatcher".
-        #
-        return _("Speech Dispatcher")
+        return guilabels.SPEECH_DISPATCHER
     getFactoryName = staticmethod(getFactoryName)
 
     def getSpeechServers():
@@ -168,21 +149,8 @@ class SpeechServer(speechserver.SpeechServer):
             speechd.CallbackType.END: speechserver.SayAllContext.COMPLETED,
            #speechd.CallbackType.INDEX_MARK:speechserver.SayAllContext.PROGRESS,
             }
-        # Translators: This string will appear in the list of
-        # available voices for the current speech engine.  %s will be
-        # replaced by the name of the current speech engine, such as
-        # "Festival default voice" or "IBMTTS default voice".  It
-        # refers to the default voice configured for given speech
-        # engine within the speech subsystem.  Apart from this item,
-        # the list will contain the names of all available "real"
-        # voices provided by the speech engine.
-        #
-        try:
-            serverId = serverId.encode("UTF-8")
-        except UnicodeDecodeError:
-            pass
 
-        self._default_voice_name = _("%s default voice") % serverId
+        self._default_voice_name = guilabels.SPEECH_DEFAULT_VOICE % serverId
         
         try:
             self._init()
@@ -193,6 +161,8 @@ class SpeechServer(speechserver.SpeechServer):
         else:
             SpeechServer._active_servers[serverId] = self
 
+        self._lastKeyEchoTime = None
+
     def _init(self):
         self._client = client = speechd.SSIPClient('Orca', component=self._id)
         client.set_priority(speechd.Priority.MESSAGE)
@@ -202,6 +172,10 @@ class SpeechServer(speechserver.SpeechServer):
         mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
         client.set_punctuation(mode)
 
+    def updateCapitalizationStyle(self):
+        """Updates the capitalization style used by the speech server."""
+        self._client.set_cap_let_recogn(settings.capitalizationStyle)
+
     def updatePunctuationLevel(self):
         """ Punctuation level changed, inform this speechServer. """
         mode = self._PUNCTUATION_MODE_MAP[settings.verbalizePunctuationStyle]
@@ -217,6 +191,8 @@ class SpeechServer(speechserver.SpeechServer):
                               "Trying to reconnect.")
                 self.reset()
                 return command(*args, **kwargs)
+            except:
+                pass
         else:
             # It is not possible tho catch the error with older SD versions. 
             return command(*args, **kwargs)
@@ -249,8 +225,6 @@ class SpeechServer(speechserver.SpeechServer):
             pass
         else:
             name = acss_family.get(speechserver.VoiceFamily.NAME)
-            if isinstance(name, unicode):
-                name = name.encode('UTF-8')
             if name != self._default_voice_name:
                 self._send_command(set_synthesis_voice, name)
             
@@ -264,7 +238,10 @@ class SpeechServer(speechserver.SpeechServer):
                 if current.get(acss_property) != value:
                     method(value)
                     current[acss_property] = value
-            elif acss_property == ACSS.FAMILY and current.get(acss_property) \
+            elif acss_property == ACSS.AVERAGE_PITCH:
+                method(5.0)
+                current[acss_property] = 5.0
+            elif acss_property == ACSS.FAMILY \
                     and acss == settings.voices[settings.DEFAULT_VOICE]:
                 # We need to explicitly reset (at least) the family.
                 # See bgo#626072.
@@ -285,19 +262,8 @@ class SpeechServer(speechserver.SpeechServer):
         Returns a text string with the punctuation symbols adjusted accordingly.
         """
 
-        # Translators: we replace the ellipses (both manual and UTF-8)
-        # with a spoken string.  The extra space you see at the beginning
-        # is because we need the speech synthesis engine to speak the
-        # new string well.  For example, "Open..." turns into
-        # "Open dot dot dot".
-        #
-        spokenEllipsis = _(" dot dot dot") + " "
+        spokenEllipsis = messages.SPOKEN_ELLIPSIS + " "
         newText = re.sub(ELLIPSIS, spokenEllipsis, oldText)
-        try:
-            newText = newText.decode("UTF-8")
-        except UnicodeEncodeError:
-            pass
-
         symbols = set(re.findall(PUNCTUATION, newText))
         for symbol in symbols:
             try:
@@ -313,34 +279,25 @@ class SpeechServer(speechserver.SpeechServer):
             charName = " %s " % chnames.getCharacterName(symbol)
             if action == punctuation_settings.PUNCTUATION_INSERT:
                 charName += symbol
-            try:
-                charName = charName.decode("UTF-8")
-            except UnicodeEncodeError:
-                pass
             newText = re.sub(symbol, charName, newText)
 
         if orca_state.activeScript:
             newText = orca_state.activeScript.utilities.adjustForDigits(newText)
 
-        try:
-            newText = newText.encode("UTF-8")
-        except UnicodeDecodeError:
-            pass
-
         return newText
 
     def _speak(self, text, acss, **kwargs):
         if isinstance(text, ACSS):
             text = ''
         text = self.__addVerbalizedPunctuation(text)
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        if orca_state.activeScript:
             text = orca_state.activeScript.\
                 utilities.adjustForPronunciation(text)
 
         # Replace no break space characters with plain spaces since some
         # synthesizers cannot handle them.  See bug #591734.
         #
-        text = text.decode("UTF-8").replace(u'\u00a0', ' ').encode("UTF-8")
+        text = text.replace('\u00a0', ' ')
 
         # Replace newline followed by full stop, since
         # this seems to crash sd, see bgo#618334.
@@ -373,9 +330,9 @@ class SpeechServer(speechserver.SpeechServer):
                         context.currentOffset = context.startOffset
                 elif t == speechserver.SayAllContext.COMPLETED:
                     context.currentOffset = context.endOffset
-                GObject.idle_add(orca_callback, context, t)
+                GLib.idle_add(orca_callback, context, t)
                 if t == speechserver.SayAllContext.COMPLETED:
-                    GObject.idle_add(self._say_all, iterator, orca_callback)
+                    GLib.idle_add(self._say_all, iterator, orca_callback)
             self._speak(context.utterance, acss, callback=callback,
                         event_types=list(self._CALLBACK_TYPE_MAP.keys()))
         return False # to indicate, that we don't want to be called again.
@@ -383,9 +340,9 @@ class SpeechServer(speechserver.SpeechServer):
     def _cancel(self):
         self._send_command(self._client.cancel)
 
-    def _change_default_speech_rate(self, decrease=False):
+    def _change_default_speech_rate(self, step, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
-        delta = settings.speechRateDelta * (decrease and -1 or +1)
+        delta = step * (decrease and -1 or +1)
         try:
             rate = acss[ACSS.RATE]
         except KeyError:
@@ -393,12 +350,13 @@ class SpeechServer(speechserver.SpeechServer):
         acss[ACSS.RATE] = max(0, min(99, rate + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech rate is now %d" % rate)
-        # Translators: This string announces speech rate change.
-        self.speak(decrease and _("slower.") or _("faster."), acss=acss)
 
-    def _change_default_speech_pitch(self, decrease=False):
+        self.speak(decrease and messages.SPEECH_SLOWER \
+                   or messages.SPEECH_FASTER, acss=acss)
+
+    def _change_default_speech_pitch(self, step, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
-        delta = settings.speechPitchDelta * (decrease and -1 or +1)
+        delta = step * (decrease and -1 or +1)
         try:
             pitch = acss[ACSS.AVERAGE_PITCH]
         except KeyError:
@@ -406,8 +364,9 @@ class SpeechServer(speechserver.SpeechServer):
         acss[ACSS.AVERAGE_PITCH] = max(0, min(9, pitch + delta))
         debug.println(debug.LEVEL_CONFIGURATION,
                       "Speech pitch is now %d" % pitch)
-        # Translators: This string announces speech pitch change.
-        self.speak(decrease and _("lower.") or _("higher."), acss=acss)
+
+        self.speak(decrease and messages.SPEECH_LOWER \
+                   or messages.SPEECH_HIGHER, acss=acss)
 
     def getInfo(self):
         return [self._SERVER_NAMES.get(self._id, self._id), self._id]
@@ -419,8 +378,9 @@ class SpeechServer(speechserver.SpeechServer):
         locale = getlocale(LC_MESSAGES)[0]
         if locale is None or locale == 'C':
             lang = None
+            dialect = None
         else:
-            lang = locale.split('_')[0]
+            lang, dialect = locale.split('_')
         voices = ((self._default_voice_name, lang, None),)
         try:
             # This command is not available with older SD versions.
@@ -428,10 +388,14 @@ class SpeechServer(speechserver.SpeechServer):
         except AttributeError:
             pass
         else:
-            voices += self._send_command(list_synthesis_voices)
+            try:
+                voices += self._send_command(list_synthesis_voices)
+            except:
+                pass
         families = [speechserver.VoiceFamily({ \
               speechserver.VoiceFamily.NAME: name,
               #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
+              speechserver.VoiceFamily.DIALECT: dialect,
               speechserver.VoiceFamily.LOCALE: lang})
                     for name, lang, dialect in voices]
         return families
@@ -440,10 +404,13 @@ class SpeechServer(speechserver.SpeechServer):
         #if interrupt:
         #    self._cancel()
 
-        if text:
-            self._speak(text, acss)
+        # "We will not interrupt a key echo in progress." (Said the comment in
+        # speech.py where these next two lines used to live. But the code here
+        # suggests we haven't been doing anything with the lastKeyEchoTime in
+        # years. TODO - JD: Dig into this and if it's truly useless, kill it.)
+        if self._lastKeyEchoTime:
+            interrupt = interrupt and (time.time() - self._lastKeyEchoTime) > 0.5
 
-    def queueText(self, text="", acss=None):
         if text:
             self._speak(text, acss)
 
@@ -455,7 +422,7 @@ class SpeechServer(speechserver.SpeechServer):
                 self._speak(utterance, acss)
 
     def sayAll(self, utteranceIterator, progressCallback):
-        GObject.idle_add(self._say_all, utteranceIterator, progressCallback)
+        GLib.idle_add(self._say_all, utteranceIterator, progressCallback)
 
     def speakCharacter(self, character, acss=None):
         self._apply_acss(acss)
@@ -468,37 +435,38 @@ class SpeechServer(speechserver.SpeechServer):
             self._send_command(self._client.char, character)
             return
 
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        if orca_state.activeScript:
             name = orca_state.activeScript.\
                 utilities.adjustForPronunciation(name)
         self.speak(name, acss)
 
     def speakKeyEvent(self, event):
-        if event.isPrintableKey():
-            # We currently only handle printable characters by Speech
-            # Dispatcher's KEY command.  For other keys, such as Ctrl, Shift
-            # etc. we prefer Orca's verbalization.
-            if event.event_string.decode("UTF-8").isupper():
-                acss = settings.voices[settings.UPPERCASE_VOICE]
-            else:
-                acss = None
-            key = self.KEY_NAMES.get(event.event_string, event.event_string)
-            self._apply_acss(acss)
-            self._send_command(self._client.key, key)
+        if event.isPrintableKey() and event.event_string.isupper():
+            acss = settings.voices[settings.UPPERCASE_VOICE]
         else:
-            return super(SpeechServer, self).speakKeyEvent(event)
+            acss = ACSS(settings.voices[settings.DEFAULT_VOICE])
+
+        event_string = event.getKeyName()
+        if orca_state.activeScript:
+            event_string = orca_state.activeScript.\
+                utilities.adjustForPronunciation(event_string)
+
+        lockingStateString = event.getLockingStateString()
+        event_string = "%s %s" % (event_string, lockingStateString)
+        self.speak(event_string, acss=acss)
+        self._lastKeyEchoTime = time.time()
 
     def increaseSpeechRate(self, step=5):
-        self._change_default_speech_rate()
+        self._change_default_speech_rate(step)
 
     def decreaseSpeechRate(self, step=5):
-        self._change_default_speech_rate(decrease=True)
+        self._change_default_speech_rate(step, decrease=True)
 
     def increaseSpeechPitch(self, step=0.5):
-        self._change_default_speech_pitch()
+        self._change_default_speech_pitch(step)
 
     def decreaseSpeechPitch(self, step=0.5):
-        self._change_default_speech_pitch(decrease=True)
+        self._change_default_speech_pitch(step, decrease=True)
 
     def stop(self):
         self._cancel()
