@@ -30,10 +30,11 @@ __copyright__ = "Copyright (c) 2008-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
-import debug
-import settings
+from . import debug
+from . import orca_state
+from . import settings
 
-from orca_i18n import _         # for gettext support
+from .orca_i18n import _         # for gettext support
 
 class TutorialGenerator:
     """Takes accessible objects and produces a tutorial string to speak
@@ -50,6 +51,8 @@ class TutorialGenerator:
         # storing the last spoken message.
         self.lastTutorial = ""
 
+        self.lastRole = None
+
         # Set up a dictionary that maps role names to functions
         # that generate tutorial strings for objects that implement that role.
         #
@@ -93,7 +96,7 @@ class TutorialGenerator:
         self.tutorialGenerators[pyatspi.ROLE_CHECK_MENU_ITEM]     = \
             self._getTutorialForCheckBox
         self.tutorialGenerators[pyatspi.ROLE_MENU_ITEM]           = \
-            self._getTutorialForPushButton
+            self._getTutorialForMenuItem
         self.tutorialGenerators[pyatspi.ROLE_RADIO_MENU_ITEM]     = \
             self._getTutorialForCheckBox
         self.tutorialGenerators[pyatspi.ROLE_SLIDER]              = \
@@ -375,6 +378,32 @@ class TutorialGenerator:
                              utterances)
         return utterances
 
+    def _getTutorialForMenuItem(self, obj, alreadyFocused, forceTutorial):
+        """Get the tutorial string for a menu item
+
+        Arguments:
+        - obj: the menu item
+        - alreadyFocused: False if object just received focus
+        - forceTutorial: used for when whereAmI really needs the tutorial string
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+        # Translators: this is the tutorial string for activating a menu item
+        msg = _("To activate press return.")
+
+        if (not alreadyFocused and self.lastTutorial != [msg]) \
+           or forceTutorial:
+            utterances.append(msg)
+
+        self._debugGenerator("_getTutorialForMenuItem",
+                             obj,
+                             alreadyFocused,
+                             utterances)
+
+        return utterances
+
     def _getTutorialForText(self, obj, alreadyFocused, forceTutorial):
         """Get the tutorial string for a text object.
 
@@ -385,6 +414,10 @@ class TutorialGenerator:
 
         Returns a list of tutorial utterances to be spoken for the object.
         """
+
+        if not obj.getState().contains(pyatspi.STATE_EDITABLE):
+            return []
+
         utterances = []
         # Translators: This is the tutorial string for when landing
         # on text fields.
@@ -519,20 +552,9 @@ class TutorialGenerator:
             cellOrder = []
             hasToggle = [ False, False ]
             for i, child in enumerate(obj):
-                try:
-                    action = child.queryAction()
-                except NotImplementedError:
-                    continue
-                else:
-                    for j in range(0, action.nActions):
-                        # Translators: this is the action name for
-                        # the 'toggle' action. It must be the same
-                        # string used in the *.po file for gail.
-                        #
-                        if action.getName(j) in ["toggle", _("toggle")]:
-                            hasToggle[i] = True
-                            break
-
+                if self._script.utilities.hasMeaningfulToggleAction(child):
+                    hasToggle[i] = True
+                    break
             if hasToggle[0] and not hasToggle[1]:
                 cellOrder = [ 1, 0 ]
             elif not hasToggle[0] and hasToggle[1]:
@@ -555,24 +577,9 @@ class TutorialGenerator:
         # state, check the NODE_CHILD_OF relation, etc.  Logged as
         # bugzilla bug 319750.]]]
         #
-        try:
-            action = obj.queryAction()
-        except NotImplementedError:
-            action = None
-        if action:
-            for i in range(0, action.nActions):
-                debug.println(debug.LEVEL_FINEST,
-                    "tutorialgenerator._getTutorialForTableCell" \
-                    + "looking at action %d" % i)
-
-                # Translators: this is the action name for
-                # the 'toggle' action. It must be the same
-                # string used in the *.po file for gail.
-                #
-                if action.getName(i) in ["toggle", _("toggle")]:
-                    utterances = self._getTutorialForCheckBox(obj,
-                                  alreadyFocused, forceTutorial)
-                    break
+        if self._script.utilities.hasMeaningfulToggleAction(obj):
+            utterances = self._getTutorialForCheckBox(
+                obj, alreadyFocused, forceTutorial)
 
         state = obj.getState()
         if state.contains(pyatspi.STATE_EXPANDABLE):
@@ -733,6 +740,21 @@ class TutorialGenerator:
 
         return utterances
 
+    def _getBindingsForHandler(self, handlerName):
+        handler = self._script.inputEventHandlers.get(handlerName)
+        if not handler:
+            return None
+
+        bindings = self._script.keyBindings.getBindingsForHandler(handler)
+        if not bindings:
+            return None
+
+        binding = bindings[0]
+        return binding.asString()
+
+    def _getModeTutorial(self, obj, alreadyFocused, forceTutorial):
+        return []
+
     def getTutorial(self, obj, alreadyFocused, forceTutorial=False):
         """Get the tutorial for an Accessible object.  This will look
         first to the specific tutorial generators and if this
@@ -751,17 +773,28 @@ class TutorialGenerator:
         if not settings.enableTutorialMessages:
             return []
 
+        if not obj == orca_state.locusOfFocus:
+            return []
+
+        utterances = []
         role = obj.getRole()
-        if role in self.tutorialGenerators:
-            generator = self.tutorialGenerators[role]
-        else:
-            generator = self._getDefaultTutorial
-        msg = generator(obj, alreadyFocused, forceTutorial)
-        utterances = [" ".join(msg)]
+        msg = self._getModeTutorial(obj, alreadyFocused, forceTutorial)
+        if not msg:
+            if role in self.tutorialGenerators:
+                generator = self.tutorialGenerators[role]
+            else:
+                generator = self._getDefaultTutorial
+            msg = generator(obj, alreadyFocused, forceTutorial)
+        elif msg == self.lastTutorial and role == self.lastRole \
+             and not forceTutorial:
+            msg = []
         if msg:
+            utterances = [" ".join(msg)]
             self.lastTutorial = msg
+            self.lastRole = role
         if forceTutorial:
             self.lastTutorial = ""
+            self.lastRole = None
 
         self._debugGenerator("getTutorial",
                              obj,
